// Copyright (c) 2013 Vittorio Romeo
// License: Academic Free License ("AFL") v. 3.0
// AFL License page: http://opensource.org/licenses/AFL-3.0

#include <iostream>
#include "SSVUtils/Encryption/Internal/Base64.h"

using namespace std;

namespace ssvu
{
	namespace Encryption
	{
		namespace Internal
		{
			static const string base64_chars{"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"};

			static inline bool is_base64(unsigned char c) { return (isalnum(c) || (c == '+') || (c == '/')); }

			string Base64Encode(unsigned char const* bytes_to_encode, unsigned int in_len)
			{
				string ret;
				int i{0}, j{0};
				unsigned char char_array_3[3], char_array_4[4];

				while(in_len--)
				{
					char_array_3[i++] = *(bytes_to_encode++);
					if(i == 3)
					{
						char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
						char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);
						char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);
						char_array_4[3] = char_array_3[2] & 0x3f;

						for(i = 0; (i <4) ; i++) ret += base64_chars[char_array_4[i]];
						i = 0;
					}
				}

				if(i)
				{
					for(j = i; j < 3; j++) char_array_3[j] = '\0';

					char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
					char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);
					char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);
					char_array_4[3] = char_array_3[2] & 0x3f;

					for(j = 0; (j < i + 1); j++) ret += base64_chars[char_array_4[j]];
					while((i++ < 3)) ret += '=';
				}

				return ret;
			}

			string Base64Decode(const string& mString)
			{
				int in_len(mString.size()), i{0}, j{0}, in_{0};
				unsigned char char_array_4[4], char_array_3[3];
				string ret;

				while(in_len-- && (mString[in_] != '=') && is_base64(mString[in_]))
				{
					char_array_4[i++] = mString[in_];
					in_++;
					if(i == 4)
					{
						for(i = 0; i <4; i++) char_array_4[i] = base64_chars.find(char_array_4[i]);

						char_array_3[0] = (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
						char_array_3[1] = ((char_array_4[1] & 0xf) << 4) + ((char_array_4[2] & 0x3c) >> 2);
						char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];

						for (i = 0; (i < 3); i++) ret += char_array_3[i];
						i = 0;
					}
				}

				if(i)
				{
					for(j = i; j <4; j++) char_array_4[j] = 0;
					for(j = 0; j <4; j++) char_array_4[j] = base64_chars.find(char_array_4[j]);

					char_array_3[0] = (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
					char_array_3[1] = ((char_array_4[1] & 0xf) << 4) + ((char_array_4[2] & 0x3c) >> 2);
					char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];

					for (j = 0; (j < i - 1); j++) ret += char_array_3[j];
				}

				return ret;
			}

			string Base64Encode(const string& mString) { return Base64Encode(reinterpret_cast<const unsigned char*>(mString.c_str()), mString.length()); }
		}
	}
}
